
.386p
.model flat, stdcall
.code

include defines.inc
include sys.inc
include hexit.inc
include funcs.inc
include bss.inc
UNICODE = 0
include w32.inc

MAINMEMBLOCKPOW		EQU	20	; 1 meg
MAINMEMBLOCKSIZE	EQU     ( 1 shl MAINMEMBLOCKPOW )


; TODO: fix mouse support
;-----------------------------------------------------------------------------
; out:  AX: 0 if ok
;       BX: -1 = 2 buttons
_initmouse:
        xor     eax,eax
        mov     ebx,-1
        ret


;-----------------------------------------------------------------------------
; out:  CX = horizontal mickeys moved
;       DX = vertical mickeys moved
_readmouse:
        xor     ecx,ecx
        xor     edx,edx
        ret


;-----------------------------------------------------------------------------
_resetmouse:
        ret


;-----------------------------------------------------------------------------
; out:  BX bit 0 = left button pressed
;       BX bit 1 = right button pressed
;       BX bit 2 = middle button pressed
_getmousebuttons:
        xor     ebx,ebx
        ret


;-----------------------------------------------------------------------------
; out:  CH = start line
;       CL = end line
;       DH = row
;       DL = column
; TODO: return size as well (somehow)
_getcursorpos:
        pushad
        call    GetConsoleScreenBufferInfo, hstdout, offset csbi
        mov     eax,csbi.dwCursorPosition
        mov     dl,al
        shr     eax,16
        mov     dh,al
        mov     [esp+5*4],edx
        popad
        ret


;-----------------------------------------------------------------------------
; in:   DH = row (0 = top)
;       DL = column (0 = leftmost)
_setcursorpos:
        pushad
        xor     eax,eax
        mov     al,dh
        shl     eax,16
        mov     al,dl
        call    SetConsoleCursorPosition, hstdout, eax
        popad
        ret


;-----------------------------------------------------------------------------
_setinsertcursor:
        pushad
        movzx   eax,curssizei
        jmp     _commoncursor

_setoverwritecursor:
        pushad
        movzx   eax,curssizeo
        jmp     _commoncursor

_setnummenufieldcursor:
        pushad
        movzx   eax,misc_nummenufieldcurssize
_commoncursor:
        imul    eax,eax,7
        cmp     eax,99          ; 100 doesn't work under win98
        jbe     @@ok
        mov     eax,99
@@ok:
        mov     cci.dwSize,eax
        mov     cci.bVisible,TRUE
        call    SetConsoleCursorInfo, hstdout, offset cci
        popad
        ret


;-----------------------------------------------------------------------------
_hidecursor:
        pushad
        mov     cci.dwSize,100          ; nt5 only hides the cursor
                                        ; if dwSize = 100 when running
                                        ; in fullscreen
        mov     cci.bVisible,FALSE
        call    SetConsoleCursorInfo, hstdout, offset cci
        popad
        ret


;-----------------------------------------------------------------------------
; out:  al = shift status bits
_getshiftkeys:
        push    ebx
        xor     al,al
        mov     bl,shiftstates
        test    bl,010h
        jz      @@ns
        or      al,3
@@ns:
        test    bl,00ch
        jz      @@nc
        or      al,4
@@nc:
        test    bl,003h
        jz      @@na
        or      al,8
@@na:
        pop     ebx
        ret

        pushad
        call    GetAsyncKeyState, VK_SHIFT
        sar     ax,15
        and     al,3
        mov     bl,al
        call    GetAsyncKeyState, VK_CONTROL
        sar     ax,15
        and     al,4
        or      bl,al
        call    GetAsyncKeyState, VK_MENU
        sar     ax,15
        and     al,8
        or      bl,al
        mov     [esp+7*4],bl
        popad
        ret


;-----------------------------------------------------------------------------
; Checks if there is a key available (in keystack or manually pressed)
; out:
;	ZF set if no key available
;	ZF clear if key available (use _getkey to fetch it)
_checkkey:
	cmp	keysinbuffer,0
	jne	_ret
        pushad
        jmp     @@lop
@@read:
        call    ReadConsoleInput, hstdin, offset inputevent, 1, offset utmp
@@lop:
        call    PeekConsoleInput, hstdin, offset inputevent, 1, offset utmp
        cmp     inputevent.EventType,KEY_EVENT
        jne     @@read
        test    utmp,1
        jz      _popadret
        mov     al,byte ptr inputevent.KeyEvent.dwControlKeyState
        mov     shiftstates,al
;        cmp     inputevent.KeyEvent.bKeyDown,0
;        je      @@read
        popad
        ret


;-----------------------------------------------------------------------------
; translates a win32 scancode to a dos one. Shift, Ctrl & Alt don't affect
; the scancode under win32
; in:   AL = win32 scancode
;       BL = shiftkeys
; out:  AL = dos scancode
_xlatewin32sc:
        push    ecx edi
        lea     edi,xlateraw
        mov     ecx,offset xlaterawend - offset xlateraw
        repne   scasb
        jecxz   @@end
        dec     edi
        test    bl,SHIFTKEY_ALT
        jz      @@ctrl
        add     edi,xlatealt - xlateraw
        jmp     @@preend
@@ctrl:
        test    bl,SHIFTKEY_CTRL
        jz      @@shift
        add     edi,xlatectrl - xlateraw
        jmp     @@preend
@@shift:
        test    bl,SHIFTKEY_SHIFT
        jz      @@end
        add     edi,xlateshift - xlateraw
@@preend:
        mov     al,[edi]
@@end:
        pop     edi ecx
        ret


;-----------------------------------------------------------------------------
; Gets the last key (or waits for a keypress if there is none).
; out:
;	AL = character
;	AH = scancode
;	BL = shiftkeys (shift, ctrl, alt. see int 16,2)
_getkey:
        cmp     keysinbuffer,0
        ja      @@popkey
@@getch:
        pushad
@@lop:
        call    ReadConsoleInput, hstdin, offset inputevent, 1, offset utmp
        cmp     inputevent.EventType,KEY_EVENT
        jne     @@lop
        mov     al,byte ptr inputevent.KeyEvent.dwControlKeyState
        mov     shiftstates,al
        popad
        xor     eax,eax
        xor     bl,bl
        cmp     inputevent.KeyEvent.bKeyDown,0
;        je      @@getch                  ; key was released
        jne     @@pressed
        mov     inputevent.KeyEvent.wVirtualScanCode,0
        mov     inputevent.KeyEvent.AsciiChar,0
@@pressed:
        call    _getshiftkeys
        test    al,SHIFTKEY_SHIFT
        jz      @@ok
        or      al,SHIFTKEY_SHIFT
@@ok:
        mov     bl,al
        shl     eax,16
        mov     al,byte ptr inputevent.KeyEvent.wVirtualScanCode
        call    _xlatewin32sc
        mov     ah,al
        mov     al,byte ptr inputevent.KeyEvent.AsciiChar
        call    _pushkey
@@popkey:
        call    _popkey
        and	eax,0ffffh
        push	eax
        sub	eax,1		; CF set if AX was zero (key released)
        pop	eax
        ret


;-----------------------------------------------------------------------------
; Waits for mousebutton to be released
; _waitbuttonrelease(buttoncode);
; buttoncode:   1 = left button
;               2 = right button
;               4 = middle button
;
_waitbuttonrelease:
        ret     4


;-----------------------------------------------------------------------------
; Waits X microseconds
; Syntax: _delay(microseconds);
_delay:
        pushad
        mov     eax,[esp+4]
        mov     ebx,1000
        xor     edx,edx
        div     ebx
        call    Sleep, eax
        popad
        ret     4


;-----------------------------------------------------------------------------
; Expands a filename with possible wildcards to the right number of files.
; In:   EDX -> Filename
; Out:  ECX = Number of files found and processed
; Note: If wildcards give more files than one then the space 128 bytes
;       after Filename is used.
_expandwildcards:
	pushad
	mov	edi,edx

	xor	al,al			; copy path to temp128bytes
	mov	ecx,128
	repne	scasb
	dec	edi
	mov	al,'\'
	std
	neg	ecx
	add	ecx,128
	repne	scasb
	cld
	jnz	@@nopath
	inc	ecx
	mov	esi,edx
@@nopath:
	lea	edi,temp128bytes
	rep	movsb
	xor	al,al
	stosb

	mov	edi,edx

	mov	ebp,1
	call	FindFirstFile, edx, offset wfd
	cmp	eax,INVALID_HANDLE_VALUE
	je	@@end
	mov	hf,eax
	xor	ebp,ebp
@@lop:
	test	wfd.dwFileAttributes, FILE_ATTRIBUTE_DIRECTORY
	jnz	@@skip
	mov	ecx,128
	lea	esi,temp128bytes
@@cplop:
	lodsb
	or	al,al
	jz	@@cpok
	stosb
	loop	@@cplop
@@cpok:
	lea	esi,wfd.cFileName
	rep	movsb
	inc	ebp
@@skip:
	call	FindNextFile, hf, offset wfd
	or	eax,eax
	jnz	@@lop
	call	FindClose, hf
@@end:
	mov	[esp+6*4],ebp		; pushed ecx
	popad
        ret


;-----------------------------------------------------------------------------
; Make full path name
_makefullpathname:
	pushad
	movzx	eax,activewindow
	movzx	eax,activefile[eax]
	shl	eax,7
	add	eax,offset filename
	call	GetFullPathName, eax, 128, eax, offset utmp
	popad
        ret


;-----------------------------------------------------------------------------
_restorevideo:
        pushad
        call    SetConsoleScreenBufferSize, hstdout, sv_csbi.dwSize
        call    SetConsoleWindowInfo, hstdout, TRUE, offset sv_csbi.srWindow
        call    SetConsoleScreenBufferSize, hstdout, sv_csbi.dwSize
        xor     edx,edx
        call    _setcursorpos
        mov     eax,sv_csbi.dwSize
        mov     ebx,eax
        and     ebx,0ffffh
        shr     eax,16
        mul     ebx
        mov     bx,sv_csbi.wAttributes
        push    eax
        call    FillConsoleOutputAttribute, hstdout, ebx, eax, 0, offset utmp
        pop     eax
        mov     ebx,' '
        call    FillConsoleOutputCharacter, hstdout, ebx, eax, 0, offset utmp
        popad
        ret


;-----------------------------------------------------------------------------
_savevideo:
        pushad
        call    GetConsoleScreenBufferInfo, hstdout, offset sv_csbi
	mov	eax,dword ptr sv_csbi.srWindow.right
        shr     eax,16
	inc	eax
	cmp	eax,140
	jbe	@@ok
	mov	eax,140
@@ok:
        mov     numberofrows,eax
        mov     videosaved,0
        movzx	eax,autodetectvideo
        dec	eax
        jz	_popadret
        mov	activevideomode,al
        mov	al,videomodes[eax-1]
        mov	numberofrows,eax
        popad
        ret


;-----------------------------------------------------------------------------
_setvideo:
        pushad
        mov     eax,numberofrows
        shl     eax,16
        mov     al,80
        push    eax
        call    SetConsoleScreenBufferSize, hstdout, eax
        mov     src.left,0
        mov     src.top,0
        mov     src.right,79
        mov     eax,numberofrows
        dec     eax
        mov     src.bottom,ax
        call    SetConsoleWindowInfo, hstdout, TRUE, offset src

; let's set the buffer size again to make sure the window gets updated
; correctly when we're setting a size smaller than the original

        pop     eax
        call    SetConsoleScreenBufferSize, hstdout, eax
        popad
        ret


;-----------------------------------------------------------------------------
_checkDPMI:
        mov     eax,1
        mov     indpmi,ax
        ret


;-----------------------------------------------------------------------------
; in:   AL = char to write
_putch:
        pushad
        mov     c,al
        call    WriteFile, hstdout, offset c, 1, offset utmp, NULL
        popad
        ret


;-----------------------------------------------------------------------------
; in:   EDX -> '$'-terminated string
;               This win32 version also accepts ASCIZ strings
_puts:
        pushad
        mov     esi,edx
        mov     ecx,-1
@@lop:
        lodsb
        inc     ecx
        cmp     al,'$'
        je      @@ok
        or      al,al
        jnz     @@lop
@@ok:
        call    WriteFile, hstdout, edx, ecx, offset utmp, NULL
        popad
        ret


;-----------------------------------------------------------------------------
; out:  ESI -> exename with full path
_getexefullpath:
	pushad
	call	GetModuleFileName, NULL, offset exefullpath, 260
	popad
	lea	esi,exefullpath
	ret


;-----------------------------------------------------------------------------
_exithexit:
        call    SetConsoleCursorInfo, hstdout, offset sv_cci
        call    GlobalFree, textscreenstart
        call    GlobalFree, pmainmem
        call    ExitProcess, 0
        ret


;-----------------------------------------------------------------------------
; out:  CF = 1 on error
_sysinit:
        pushad
        call    SetConsoleTitle, offset atitle

        call    GetStdHandle, STD_INPUT_HANDLE
        mov     hstdin,eax
        call    GetStdHandle, STD_OUTPUT_HANDLE
        mov     hstdout,eax

        call    GetConsoleCursorInfo, hstdout, offset sv_cci

        call    GlobalAlloc, GMEM_FIXED, 65536
        or      eax,eax
        jz      _popadstcret
        mov     textscreenstart,eax

	call	GetProcessHeap
	call	HeapAlloc, eax, NULL, MAINMEMBLOCKSIZE  ; alloc mem block

        mov     pmainmemtop,eax
        mov     freehimemory,eax
        or      eax,eax
        jz      _popadstcret
        add     pmainmemtop,MAINMEMBLOCKSIZE
	mov	freehimemory,MAINMEMBLOCKSIZE
        mov     pmainmem,eax
        mov     pfreemem,eax
        popad
        clc
        ret


;-----------------------------------------------------------------------------
; in:	EAX = additional size needed
; out:	CF  = 1 on error
; note: WARNING! Memory pointers may have changed after a call to this
;	routine. Therefore, always reload pointers after calling it!
_reallocmainmem:
	pushad
	add	eax,pmainmemtop
	sub	eax,pmainmem

	shr	eax,MAINMEMBLOCKPOW
	inc	eax
	shl	eax,MAINMEMBLOCKPOW

	mov	ebp,eax
	call	GetProcessHeap
	call	HeapReAlloc, eax, NULL, pmainmem, ebp
	or	eax,eax
	jz	_popadstcret

	mov	ebp,pmainmemtop
	sub	ebp,pmainmem		; old size

	sub	eax,pmainmem
	mov	ecx,MAXFILES
@@lop:
	add	filebufferptr[ecx*4-4],eax
	loop	@@lop

	add	pkeybuffer,eax
	add	precbuffer,eax
	add	plineoffset,eax
	add	pfreemem,eax
	add	pmainmem,eax
	add	pmainmemtop,eax
	add	clipptr,eax
	add	mark1ptr,eax
	add	mark2ptr,eax

	call	GetProcessHeap
	call	HeapSize, eax, NULL, pmainmem
	sub	eax,ebp
	add	freehimemory,eax
	add	pmainmemtop,eax
	add	maxclipsize,eax

	popad
	clc
	ret


;-----------------------------------------------------------------------------
_allocmainmem:
        mov     eax,pmainmemtop
        mov	freehimemory,eax
        xchg    pfreemem,eax
        sub	freehimemory,eax
        ret


;-----------------------------------------------------------------------------
; out:	EAX = max # of free bytes available for allocation
_getmaxfreemem:
	pushad
	call	GlobalMemoryStatus, offset mst
	popad
	mov	eax,mst.dwAvailPhys
	add	eax,mst.dwAvailPageFile
	add	eax,freehimemory
	cmp	eax,mst.dwAvailVirtual
	jbe	_ret
	mov	eax,mst.dwAvailVirtual
	add	eax,freehimemory
	ret


;-----------------------------------------------------------------------------
_getlomem:
_gethimem:
        add     eax,pfreemem
        cmp     eax,pmainmemtop
        ja      @@err
        xchg    eax,pfreemem
        clc
        ret
@@err:
        sub     eax,pmainmemtop
        stc
        ret


;-----------------------------------------------------------------------------
_getmainmemtop:
        mov     eax,pmainmemtop
        ret


;-----------------------------------------------------------------------------
; ESI -> cmdline
; ECX == length of cmdline
_getcmdline:
        pushad
        call    GetCommandLine
        
        mov     esi,eax
@@lop:
        lodsb
        cmp     al,' '
        je      @@ok
        or      al,al
        jnz     @@lop
@@ok:
        mov     edi,esi
        mov     ecx,-1
        mov     al,0
        repne   scasb
        sub     edi,esi
        dec     edi
        mov     [esp+6*4],edi
        mov     [esp+1*4],esi
        popad
        ret


;-----------------------------------------------------------------------------
; in:   EDX -> filename
; out:  ECX = attribs
;	CF = 1 on error
_getfileattribs:
        pushad
        call    GetFileAttributes, edx
        mov     [esp+6*4],eax
        add	eax,1			; retval == -1 if function fails
        popad
        ret


;-----------------------------------------------------------------------------
; in:   EDX -> filename
; out:  ECX = new attribs
;	CF = 1 on error
_setfileattribs:
        and     ecx,0ffffh
        pushad
        call    SetFileAttributes, edx, ecx
        sub	eax,1			; retval == 0 if function fails
        popad
        ret


;-----------------------------------------------------------------------------
; in:   EDX -> filename
; out:  CF = 1 on error
_createfile:
        pushad
        call    CreateFile, edx, GENERIC_READ OR GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, NULL
        mov     fhandle,eax
        cmp     eax,INVALID_HANDLE_VALUE
        popad
        je      _stcret
        clc
        ret


;-----------------------------------------------------------------------------
; in:   EDX -> filename
; out:  CF = 1 on error
_openfile:
        pushad
        call    CreateFile, edx, GENERIC_READ OR GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL
        mov     fhandle,eax
        cmp     eax,INVALID_HANDLE_VALUE
        popad
        je      _stcret
        clc
        ret


;-----------------------------------------------------------------------------
; in:   EDX -> filename
; out:  CF = 1 on error
_openfilereadonly:
        pushad
        call    CreateFile, edx, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL
        mov     fhandle,eax
        cmp     eax,INVALID_HANDLE_VALUE
        popad
        je      _stcret
        clc
        ret


;-----------------------------------------------------------------------------
; out:  CF = 1 on error
_closefile:
        pushad
        call    CloseHandle, fhandle
        or      eax,eax
        popad
        jz      _stcret
        clc
        ret


;-----------------------------------------------------------------------------
; in:   EDX -> filename
; out:  CF = 1 on error
_deletefile:
        pushad
        call    DeleteFile, edx
        or      eax,eax
        popad
        jz      _stcret
        clc
        ret


;-----------------------------------------------------------------------------
; in:   EAX = offset to move to
;       BL  = from: 0 = SEEK_SET, 1 = SEEK_CUR, 2 = SEEK_END
; out:  CF  = 1 on error
;       EAX = new offset from BOF
_lseekfile:
        pushad
        movzx   ebx,bl
        call    SetFilePointer, fhandle, eax, NULL, ebx
        mov     [esp+7*4],eax
        popad
        cmp     eax,-1
        je      _stcret
        clc
        ret

;-----------------------------------------------------------------------------
; out:  CF  = 1 on error
;       EAX = filesize
_filesize:
        pushad
        call    GetFileSize, fhandle, NULL
        mov     [esp+7*4],eax
        popad
        cmp     eax,-1
        je      _stcret
        clc
        ret


;-----------------------------------------------------------------------------
; in:   EDX -> buffer to read to
;       ECX = # of bytes to read
; out:  CF  = 1 on error
;       EAX = # of bytes read
_readfile:
        pushad
        call    ReadFile, fhandle, edx, ecx, offset utmp, NULL
        or      eax,eax
        popad
        mov     eax,utmp
;        jz      _stcret
	jz	@@err
        clc
        ret

; TODO: remove this
@@err:
	pushad
	call	GetLastError
	add	eax,1000000
	call	_printdec_dwordt
	popad
	stc
	ret


;-----------------------------------------------------------------------------
; in:   EDX -> buffer to write from
;       ECX = # of bytes to write
; out:  CF  = 1 on error
;       EAX = # of bytes written
_writefile:
        pushad
        call    WriteFile, fhandle, edx, ecx, offset utmp, NULL
        or      eax,eax
        popad
        mov     eax,utmp
        jz      _stcret
        clc
        ret


;-----------------------------------------------------------------------------
; in:	EAX == index of first row to update (0 == first)
;	ECX == # of rows to update
_updaterows:
	pushad
	shl	eax,16
	mov	dword ptr [put_sr.left],eax

	shl	ecx,16
        add     ecx,50h                         ; 80 columns wide

        lea     ebx,[eax+ecx-10001h]            ; x2--; y2--
	mov	dword ptr [put_sr.right],ebx

	shr	eax,16-6
	lea	eax,[eax*4+eax]
	add	eax,[textscreenstart]

	call	WriteConsoleOutput, hstdout, eax, ecx, 0, offset put_sr
	popad
	ret


;-----------------------------------------------------------------------------
; No absolute memory references are allowed, use a register for
; indirection.
;
; Exit helper programs with a ret
;
; Initial register states:
;       eax = ?
;       ebx = ?
;       ecx = length in bytes of current file
;       edx = ?
;       esi = base address of current file
;       edi = ?
;       esp = stack
;       ebp = base address of helper program
;
_runhelper:
        pushad
        call    GlobalAlloc, GMEM_FIXED, 65536
        or      eax,eax
        jz      _popadstcret
        mov     helperbase,eax
        mov     ebp,eax
        lea     edx,helperfname
        call    _openfile
        jc      @@err1
        call    _filesize
        jc      @@err2
        mov     ecx,eax
        mov     edx,ebp
        call    _readfile
        jc      @@err2
        call    _closefile
        movzx   ebx,activewindow
        movzx   eax,activefile[ebx]
        shl     ebx,2
        mov     ecx,filesize[eax*4]
        mov     esi,filebufferptr[eax*4]
;        add     esi,filebufferpos[eax*8+ebx]
        call    ebp
        call    GlobalFree, helperbase
        call    _putbothscreens&cursor
        popad
        clc
        ret

@@err2:
        call    _closefile
@@err1:
        call    GlobalFree, helperbase
        popad
        stc
        ret

helperfname     db      'morse.hxt',0


;-----------------------------------------------------------------------------
; dummies
_slice:
        ret


;-----------------------------------------------------------------------------
_popad&ret:
_popadret:
        popad
        ret

_popadstcret:
        popad
        stc
        ret

_popadclcret:
        popad
        clc
        ret

_clcret:
        clc
        ret

_stcret:
        stc
_ret:
        ret


;-----------------------------------------------------------------------------
.data
atitle  db      'HexIt ',version,0

xlateraw        label   byte
        db      KEY_HOME
        db      KEY_END
        db      KEY_LEFT
        db      KEY_RIGHT
        db      KEY_UP
        db      KEY_DOWN
        db      KEY_PAGEUP
        db      KEY_PAGEDOWN
        db      KEY_F1
        db      KEY_F2
        db      KEY_F3
        db      KEY_F4
        db      KEY_F5
        db      KEY_F6
        db      KEY_F7
        db      KEY_F8
        db      KEY_F9
        db      KEY_F10
        db      KEY_F11
        db      KEY_F12
        db      KEY_TAB
        db      KEY_INS
        db      KEY_DEL
        db      KEY_MINUS
        db      KEY_PLUS
        db      KEY_MULTIPLY
        db      KEY_DIVIDE
        db      KEY_GRAY_ENTER
        db      KEY_1
        db      KEY_2
        db      KEY_3
        db      KEY_4
        db      KEY_5
        db      KEY_6
        db      KEY_7
        db      KEY_8
        db      KEY_9
        db      KEY_0

        db      0       ; sentinel
xlaterawend     label   byte

xlateshift      label   byte
        db      KEY_HOME
        db      KEY_END
        db      KEY_LEFT
        db      KEY_RIGHT
        db      KEY_UP
        db      KEY_DOWN
        db      KEY_PAGEUP
        db      KEY_PAGEDOWN
        db      KEY_SHIFT_F1
        db      KEY_SHIFT_F2
        db      KEY_SHIFT_F3
        db      KEY_SHIFT_F4
        db      KEY_SHIFT_F5
        db      KEY_SHIFT_F6
        db      KEY_SHIFT_F7
        db      KEY_SHIFT_F8
        db      KEY_SHIFT_F9
        db      KEY_SHIFT_F10
        db      KEY_SHIFT_F11
        db      KEY_SHIFT_F12
        db      KEY_TAB
        db      KEY_INS
        db      KEY_DEL
        db      KEY_MINUS
        db      KEY_PLUS
        db      KEY_MULTIPLY
        db      KEY_DIVIDE
        db      KEY_GRAY_ENTER
        db      KEY_1
        db      KEY_2
        db      KEY_3
        db      KEY_4
        db      KEY_5
        db      KEY_6
        db      KEY_7
        db      KEY_8
        db      KEY_9
        db      KEY_0

xlatectrl       label   byte
        db      KEY_CTRL_HOME
        db      KEY_CTRL_END
        db      KEY_CTRL_LEFT
        db      KEY_CTRL_RIGHT
        db      KEY_CTRL_UP
        db      KEY_CTRL_DOWN
        db      KEY_CTRL_PAGEUP
        db      KEY_CTRL_PAGEDOWN
        db      KEY_CTRL_F1
        db      KEY_CTRL_F2
        db      KEY_CTRL_F3
        db      KEY_CTRL_F4
        db      KEY_CTRL_F5
        db      KEY_CTRL_F6
        db      KEY_CTRL_F7
        db      KEY_CTRL_F8
        db      KEY_CTRL_F9
        db      KEY_CTRL_F10
        db      KEY_CTRL_F11
        db      KEY_CTRL_F12
        db      KEY_CTRL_TAB
        db      KEY_CTRL_INS
        db      KEY_CTRL_DEL
        db      KEY_CTRL_MINUS
        db      KEY_CTRL_PLUS
        db      KEY_CTRL_MULTIPLY
        db      KEY_CTRL_DIVIDE
        db      KEY_GRAY_ENTER
        db      KEY_1
        db      KEY_2
        db      KEY_3
        db      KEY_4
        db      KEY_5
        db      KEY_6
        db      KEY_7
        db      KEY_8
        db      KEY_9
        db      KEY_0

xlatealt        label   byte
        db      KEY_ALT_HOME
        db      KEY_ALT_END
        db      KEY_ALT_LEFT
        db      KEY_ALT_RIGHT
        db      KEY_ALT_UP
        db      KEY_ALT_DOWN
        db      KEY_ALT_PAGEUP
        db      KEY_ALT_PAGEDOWN
        db      KEY_ALT_F1
        db      KEY_ALT_F2
        db      KEY_ALT_F3
        db      KEY_ALT_F4
        db      KEY_ALT_F5
        db      KEY_ALT_F6
        db      KEY_ALT_F7
        db      KEY_ALT_F8
        db      KEY_ALT_F9
        db      KEY_ALT_F10
        db      KEY_ALT_F11
        db      KEY_ALT_F12
        db      KEY_TAB
        db      KEY_ALT_INS
        db      KEY_ALT_DEL
        db      KEY_MINUS
        db      KEY_PLUS
        db      KEY_MULTIPLY
        db      KEY_ALT_DIVIDE
        db      KEY_ALT_GRAY_ENTER
        db      KEY_ALT_1
        db      KEY_ALT_2
        db      KEY_ALT_3
        db      KEY_ALT_4
        db      KEY_ALT_5
        db      KEY_ALT_6
        db      KEY_ALT_7
        db      KEY_ALT_8
        db      KEY_ALT_9
        db      KEY_ALT_0


pmainmem        dd      	0
pfreemem        dd      	?
pmainmemtop     dd      	?
hstdin          dd      	?
hstdout         dd      	?
inputevent      INPUT_RECORD    ?
shiftstates     db      	?
utmp            dd      	?
c               db      	?
fhandle         dd      	?
src             SMALL_RECT                      ?
cci             CONSOLE_CURSOR_INFO             ?
csbi            CONSOLE_SCREEN_BUFFER_INFO      ?

; _savevideo info
sv_csbi         CONSOLE_SCREEN_BUFFER_INFO      ?
sv_cci          CONSOLE_CURSOR_INFO             ?

put_sr		SMALL_RECT	?

wfd		WIN32_FIND_DATA	?
hf		dd		?

mst		MEMORYSTATUS	?

helperbase      dd      	?


end
